@@ -29,22 +29,24 @@ __license__ = "LGPL"
 from . import cmdnames
 from . import debug
 from . import input_event
+from . import input_event_manager
 from . import keybindings
 from . import messages
 from . import settings_manager
 
+from .ax_text import AXText
 
 class CaretNavigation:
     """Implements the caret navigation support available to scripts."""
 
-    def __init__(self, script):
-        if not (script and script.app):
-            msg = "CARET NAVIGATION: Caret navigation requires a script and app."
-            debug.printMessage(debug.LEVEL_INFO, msg)
+    def __init__(self):
+        # To make it possible for focus mode to suspend this navigation without
+        # changing the user's preferred setting.
+        self._suspended = False
 
-        self._script = script
-        self._handlers = self._setup_handlers()
-        self._bindings = self._setup_bindings()
+        self._handlers = self.get_handlers(True)
+        self._bindings = keybindings.KeyBindings()
+        self._last_input_event = None
 
     def handles_navigation(self, handler):
         """Returns True if handler is a navigation command."""
@@ -52,193 +54,289 @@ class CaretNavigation:
         if handler not in self._handlers.values():
             return False
 
-        if handler.function == self._toggle_enabled:
+        if handler.function == self.toggle_enabled:
             return False
 
         return True
 
-    def get_bindings(self):
+    def get_bindings(self, refresh=False, is_desktop=True):
         """Returns the caret-navigation keybindings."""
 
+        if refresh:
+            msg = "CARET NAVIGATION: Refreshing bindings."
+            debug.printMessage(debug.LEVEL_INFO, msg, True, True)
+            self._setup_bindings()
+        elif self._bindings.is_empty():
+            self._setup_bindings()
+
         return self._bindings
 
-    def get_handlers(self):
+    def get_handlers(self, refresh=False):
         """Returns the caret-navigation handlers."""
 
+        if refresh:
+            msg = "CARET NAVIGATION: Refreshing handlers."
+            debug.printMessage(debug.LEVEL_INFO, msg, True, True)
+            self._setup_handlers()
+
         return self._handlers
 
     def _setup_handlers(self):
-        """Sets up and returns the caret-navigation input event handlers."""
+        """Sets up the caret-navigation input event handlers."""
 
-        handlers = {}
+        self._handlers = {}
 
-        if not (self._script and self._script.app):
-            return handlers
-
-        handlers["toggle_enabled"] = \
+        self._handlers["toggle_enabled"] = \
             input_event.InputEventHandler(
-                self._toggle_enabled,
-                cmdnames.CARET_NAVIGATION_TOGGLE)
+                self.toggle_enabled,
+                cmdnames.CARET_NAVIGATION_TOGGLE,
+                enabled = not self._suspended)
 
-        handlers["next_character"] = \
+        enabled = settings_manager.get_manager().get_setting('caretNavigationEnabled') \
+            and not self._suspended
+
+        self._handlers["next_character"] = \
             input_event.InputEventHandler(
                 self._next_character,
-                cmdnames.CARET_NAVIGATION_NEXT_CHAR)
+                cmdnames.CARET_NAVIGATION_NEXT_CHAR,
+                enabled = enabled)
 
-        handlers["previous_character"] = \
+        self._handlers["previous_character"] = \
             input_event.InputEventHandler(
                 self._previous_character,
-                cmdnames.CARET_NAVIGATION_PREV_CHAR)
+                cmdnames.CARET_NAVIGATION_PREV_CHAR,
+                enabled = enabled)
 
-        handlers["next_word"] = \
+        self._handlers["next_word"] = \
             input_event.InputEventHandler(
                 self._next_word,
-                cmdnames.CARET_NAVIGATION_NEXT_WORD)
+                cmdnames.CARET_NAVIGATION_NEXT_WORD,
+                enabled = enabled)
 
-        handlers["previous_word"] = \
+        self._handlers["previous_word"] = \
             input_event.InputEventHandler(
                 self._previous_word,
-                cmdnames.CARET_NAVIGATION_PREV_WORD)
+                cmdnames.CARET_NAVIGATION_PREV_WORD,
+                enabled = enabled)
 
-        handlers["next_line"] = \
+        self._handlers["next_line"] = \
             input_event.InputEventHandler(
                 self._next_line,
-                cmdnames.CARET_NAVIGATION_NEXT_LINE)
+                cmdnames.CARET_NAVIGATION_NEXT_LINE,
+                enabled = enabled)
 
-        handlers["previous_line"] = \
+        self._handlers["previous_line"] = \
             input_event.InputEventHandler(
                 self._previous_line,
-                cmdnames.CARET_NAVIGATION_PREV_LINE)
+                cmdnames.CARET_NAVIGATION_PREV_LINE,
+                enabled = enabled)
 
-        handlers["start_of_file"] = \
+        self._handlers["start_of_file"] = \
             input_event.InputEventHandler(
                 self._start_of_file,
-                cmdnames.CARET_NAVIGATION_FILE_START)
+                cmdnames.CARET_NAVIGATION_FILE_START,
+                enabled = enabled)
 
-        handlers["end_of_file"] = \
+        self._handlers["end_of_file"] = \
             input_event.InputEventHandler(
                 self._end_of_file,
-                cmdnames.CARET_NAVIGATION_FILE_END)
+                cmdnames.CARET_NAVIGATION_FILE_END,
+                enabled = enabled)
 
-        handlers["start_of_line"] = \
+        self._handlers["start_of_line"] = \
             input_event.InputEventHandler(
                 self._start_of_line,
-                cmdnames.CARET_NAVIGATION_LINE_START)
+                cmdnames.CARET_NAVIGATION_LINE_START,
+                enabled = enabled)
 
-        handlers["end_of_line"] = \
+        self._handlers["end_of_line"] = \
             input_event.InputEventHandler(
                 self._end_of_line,
-                cmdnames.CARET_NAVIGATION_LINE_END)
+                cmdnames.CARET_NAVIGATION_LINE_END,
+                enabled = enabled)
 
-        return handlers
+        msg = f"CARET NAVIGATION: Handlers set up. Suspended: {self._suspended}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _setup_bindings(self):
-        """Sets up and returns the caret-navigation key bindings."""
+        """Sets up the caret-navigation key bindings."""
 
-        bindings = keybindings.KeyBindings()
+        self._bindings = keybindings.KeyBindings()
 
-        if not (self._script and self._script.app):
-            return bindings
-
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "F12",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
-                self._handlers.get("toggle_enabled")))
+                self._handlers.get("toggle_enabled"),
+                1,
+                not self._suspended))
 
-        bindings.add(
+        enabled = settings_manager.get_manager().get_setting('caretNavigationEnabled') \
+            and not self._suspended
+
+        self._bindings.add(
             keybindings.KeyBinding(
                 "Right",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("next_character")))
+                self._handlers.get("next_character"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "Left",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("previous_character")))
+                self._handlers.get("previous_character"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "Right",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.CTRL_MODIFIER_MASK,
-                self._handlers.get("next_word")))
+                self._handlers.get("next_word"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "Left",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.CTRL_MODIFIER_MASK,
-                self._handlers.get("previous_word")))
+                self._handlers.get("previous_word"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "Down",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("next_line")))
+                self._handlers.get("next_line"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "Up",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("previous_line")))
+                self._handlers.get("previous_line"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "End",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("end_of_line")))
+                self._handlers.get("end_of_line"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "Home",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
-                self._handlers.get("start_of_line")))
+                self._handlers.get("start_of_line"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "End",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.CTRL_MODIFIER_MASK,
-                self._handlers.get("end_of_file")))
+                self._handlers.get("end_of_file"),
+                1,
+                enabled))
 
-        bindings.add(
+        self._bindings.add(
             keybindings.KeyBinding(
                 "Home",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.CTRL_MODIFIER_MASK,
-                self._handlers.get("start_of_file")))
+                self._handlers.get("start_of_file"),
+                1,
+                enabled))
 
-        return bindings
+        # This pulls in the user's overrides to alternative keys.
+        self._bindings = settings_manager.get_manager().override_key_bindings(
+            self._handlers, self._bindings, False)
 
-    @staticmethod
-    def _toggle_enabled(script, event):
+        msg = f"CARET NAVIGATION: Bindings set up. Suspended: {self._suspended}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+    def last_input_event_was_navigation_command(self):
+        """Returns true if the last input event was a navigation command."""
+
+        manager = input_event_manager.get_manager()
+        result = manager.last_event_equals_or_is_release_for_event(self._last_input_event)
+        if self._last_input_event is not None:
+            string = self._last_input_event.as_single_line_string()
+        else:
+            string = "None"
+
+        msg = f"CARET NAVIGATION: Last navigation event ({string}) is last key event: {result}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        return result
+
+    def refresh_bindings_and_grabs(self, script, reason=""):
+        """Refreshes caret navigation bindings and grabs for script."""
+
+        msg = "CARET NAVIGATION: Refreshing bindings and grabs"
+        if reason:
+            msg += f": {reason}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+        for binding in self._bindings.key_bindings:
+            script.key_bindings.remove(binding, include_grabs=True)
+
+        self._handlers = self.get_handlers(True)
+        self._bindings = self.get_bindings(True)
+
+        for binding in self._bindings.key_bindings:
+            script.key_bindings.add(binding, include_grabs=not self._suspended)
+
+    def toggle_enabled(self, script, event):
         """Toggles caret navigation."""
 
         if not event:
             return False
 
-        _settings_manager = settings_manager.getManager()
-        enabled = not _settings_manager.getSetting('caretNavigationEnabled')
+        _settings_manager = settings_manager.get_manager()
+        enabled = not _settings_manager.get_setting('caretNavigationEnabled')
         if enabled:
             string = messages.CARET_CONTROL_ORCA
         else:
             string = messages.CARET_CONTROL_APP
 
         script.presentMessage(string)
-        _settings_manager.setSetting('caretNavigationEnabled', enabled)
+        _settings_manager.set_setting('caretNavigationEnabled', enabled)
+        self._last_input_event = None
+        self.refresh_bindings_and_grabs(script, "toggling caret navigation")
         return True
 
-    @staticmethod
-    def _next_character(script, event):
+    def suspend_commands(self, script, suspended, reason=""):
+        """Suspends caret navigation independent of the enabled setting."""
+
+        if suspended == self._suspended:
+            return
+
+        msg = f"CARET NAVIGATION: Commands suspended: {suspended}"
+        if reason:
+            msg += f": {reason}"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+        self._suspended = suspended
+        self.refresh_bindings_and_grabs(script, f"Suspended changed to {suspended}")
+
+    def _next_character(self, script, event):
         """Moves to the next character."""
 
         if not event:
@@ -248,13 +346,14 @@ class CaretNavigation:
         if not obj:
             return False
 
+        self._last_input_event = event
         script.utilities.setCaretPosition(obj, offset)
-        script.updateBraille(obj)
+        script.presentationInterrupt()
+        script.update_braille(obj)
         script.sayCharacter(obj)
         return True
 
-    @staticmethod
-    def _previous_character(script, event):
+    def _previous_character(self, script, event):
         """Moves to the previous character."""
 
         if not event:
@@ -264,13 +363,14 @@ class CaretNavigation:
         if not obj:
             return False
 
+        self._last_input_event = event
         script.utilities.setCaretPosition(obj, offset)
-        script.updateBraille(obj)
+        script.presentationInterrupt()
+        script.update_braille(obj)
         script.sayCharacter(obj)
         return True
 
-    @staticmethod
-    def _next_word(script, event):
+    def _next_word(self, script, event):
         """Moves to the next word."""
 
         if not event:
@@ -285,13 +385,14 @@ class CaretNavigation:
         if string and string[-1].isspace():
             end -= 1
 
+        self._last_input_event = event
         script.utilities.setCaretPosition(obj, end)
-        script.updateBraille(obj)
+        script.presentationInterrupt()
+        script.update_braille(obj)
         script.sayWord(obj)
         return True
 
-    @staticmethod
-    def _previous_word(script, event):
+    def _previous_word(self, script, event):
         """Moves to the previous word."""
 
         if not event:
@@ -302,22 +403,23 @@ class CaretNavigation:
         if not contents:
             return False
 
+        self._last_input_event = event
         obj, start = contents[0][0], contents[0][1]
         script.utilities.setCaretPosition(obj, start)
-        script.updateBraille(obj)
+        script.presentationInterrupt()
+        script.update_braille(obj)
         script.sayWord(obj)
         return True
 
-    @staticmethod
-    def _next_line(script, event):
+    def _next_line(self, script, event):
         """Moves to the next line."""
 
         if not event:
             return False
 
         if script.inSayAll():
-            _settings_manager = settings_manager.getManager()
-            if _settings_manager.getSetting('rewindAndFastForwardInSayAll'):
+            _settings_manager = settings_manager.get_manager()
+            if _settings_manager.get_setting('rewindAndFastForwardInSayAll'):
                 msg = "CARET NAVIGATION: inSayAll and rewindAndFastforwardInSayAll is enabled"
                 debug.printMessage(debug.LEVEL_INFO, msg)
                 return True
@@ -331,22 +433,23 @@ class CaretNavigation:
         if not contents:
             return False
 
+        self._last_input_event = event
         obj, start = contents[0][0], contents[0][1]
         script.utilities.setCaretPosition(obj, start)
+        script.presentationInterrupt()
         script.speakContents(contents, priorObj=line[-1][0])
         script.displayContents(contents)
         return True
 
-    @staticmethod
-    def _previous_line(script, event):
+    def _previous_line(self, script, event):
         """Moves to the previous line."""
 
         if not event:
             return False
 
         if script.inSayAll():
-            _settings_manager = settings_manager.getManager()
-            if _settings_manager.getSetting('rewindAndFastForwardInSayAll'):
+            _settings_manager = settings_manager.get_manager()
+            if _settings_manager.get_setting('rewindAndFastForwardInSayAll'):
                 msg = "CARET NAVIGATION: inSayAll and rewindAndFastforwardInSayAll is enabled"
                 debug.printMessage(debug.LEVEL_INFO, msg)
                 return True
@@ -356,14 +459,15 @@ class CaretNavigation:
         if not contents:
             return False
 
+        self._last_input_event = event
         obj, start = contents[0][0], contents[0][1]
         script.utilities.setCaretPosition(obj, start)
+        script.presentationInterrupt()
         script.speakContents(contents)
         script.displayContents(contents)
         return True
 
-    @staticmethod
-    def _start_of_line(script, event):
+    def _start_of_line(self, script, event):
         """Moves to the start of the line."""
 
         if not event:
@@ -374,14 +478,15 @@ class CaretNavigation:
         if not (line and line[0]):
             return False
 
+        self._last_input_event = event
         obj, start = line[0][0], line[0][1]
         script.utilities.setCaretPosition(obj, start)
+        script.presentationInterrupt()
         script.sayCharacter(obj)
         script.displayContents(line)
         return True
 
-    @staticmethod
-    def _end_of_line(script, event):
+    def _end_of_line(self, script, event):
         """Moves to the end of the line."""
 
         if not event:
@@ -396,13 +501,14 @@ class CaretNavigation:
         if string.strip() and string[-1].isspace():
             end -= 1
 
+        self._last_input_event = event
         script.utilities.setCaretPosition(obj, end)
+        script.presentationInterrupt()
         script.sayCharacter(obj)
         script.displayContents(line)
         return True
 
-    @staticmethod
-    def _start_of_file(script, event):
+    def _start_of_file(self, script, event):
         """Moves to the start of the file."""
 
         if not event:
@@ -414,14 +520,15 @@ class CaretNavigation:
         if not contents:
             return False
 
+        self._last_input_event = event
         obj, offset = contents[0][0], contents[0][1]
         script.utilities.setCaretPosition(obj, offset)
+        script.presentationInterrupt()
         script.speakContents(contents)
         script.displayContents(contents)
         return True
 
-    @staticmethod
-    def _end_of_file(script, event):
+    def _end_of_file(self, script, event):
         """Moves to the end of the file."""
 
         if not event:
@@ -435,11 +542,7 @@ class CaretNavigation:
         tokens = ["CARET NAVIGATION: Last object in", document, "is", obj]
         debug.printTokens(debug.LEVEL_INFO, tokens, True)
 
-        offset = 0
-        text = script.utilities.queryNonEmptyText(obj)
-        if text:
-            offset = text.characterCount - 1
-
+        offset = max(0, AXText.get_character_count(obj) - 1)
         while obj:
             lastobj, lastoffset = script.utilities.nextContext(obj, offset)
             if not lastobj:
@@ -450,8 +553,10 @@ class CaretNavigation:
         if not contents:
             return False
 
+        self._last_input_event = event
         obj, offset = contents[-1][0], contents[-1][2]
         script.utilities.setCaretPosition(obj, offset)
+        script.presentationInterrupt()
         script.speakContents(contents)
         script.displayContents(contents)
         return True
