@@ -32,25 +32,19 @@ __license__   = "LGPL"
 
 from .orca_i18n import _
 
-# Translators: this command will move the mouse pointer to the current item
-#  without clicking on it.             
+# Translators: this command will move the mouse pointer to the current item,
+# typically a widget, without clicking on it.
 ROUTE_POINTER_TO_ITEM = _("Route the pointer to the current item")
 
-# Translators: the 'flat review' feature of Orca allows the blind user to
-# explore the text in a window in a 2D fashion.  That is, Orca treats all
-# the text from all objects in a window (e.g., buttons, labels, etc.) as a
-# sequence of words in a sequence of lines.  The flat review feature allows
-# the user to explore this text by the {previous,next} {line,word,character}.
-# Left click means to generate a left mouse button click on the current item.
-LEFT_CLICK_REVIEW_ITEM = _("Perform left click on current flat review item")
+# Translators: Orca has a command to synthesize mouse events. This string
+# describes the Orca command to generate a left mouse button click on the
+# current item, typically a widget.
+LEFT_CLICK_REVIEW_ITEM = _("Perform left click on current item")
 
-# Translators: the 'flat review' feature of Orca allows the blind user to
-# explore the text in a window in a 2D fashion.  That is, Orca treats all
-# the text from all objects in a window (e.g., buttons, labels, etc.) as a
-# sequence of words in a sequence of lines.  The flat review feature allows
-# the user to explore this text by the {previous,next} {line,word,character}.
-# Right click means to generate a right mouse button click on the current item.
-RIGHT_CLICK_REVIEW_ITEM = _("Perform right click on current flat review item")
+# Translators: Orca has a command to synthesize mouse events. This string
+# describes the Orca command to generate a right mouse button click on the
+# current item, typically a widget.
+RIGHT_CLICK_REVIEW_ITEM = _("Perform right click on current item")
 
 # Translators: the Orca "SayAll" command allows the user to press a key and have
 # the entire document in a window be automatically spoken to the user. If the
@@ -449,6 +443,20 @@ DECREASE_SPEECH_VOLUME = _("Decrease the speech volume")
 #  We call it 'silencing'.
 TOGGLE_SPEECH = _("Toggle the silencing of speech")
 
+# Translators: Orca has a sleep mode which causes Orca to essentially behave as
+# if it were not running for a given application. Some use cases include self-
+# voicing apps with associated commands (e.g. ChromeVox) and VMs. In the former
+# case, the self-voicing app is expected to provide all needed commands as well
+# as speech and braille. In the latter case, we want to ensure that Orca's
+# commands and speech/braille do not interfere with that of the VM and any
+# screen reader being used in that VM. Thus when an application is being used
+# in sleep mode, nearly all Orca commands become unbound/free, and nothing is
+# spoken or brailled. But if the user toggles sleep mode off or switches to
+# another application window, Orca commands, speech, and braille immediately
+# resume working. This string is the command which toggles sleep mode on/off
+# for the app being used at the time the command is given.
+TOGGLE_SLEEP_MODE = _("Toggle sleep mode for the current application")
+
 # Translators: Orca's verbosity levels control how much (or how little)
 # Orca will speak when presenting objects as the user navigates within
 # applications and reads content. The levels can be toggled via command.
@@ -558,6 +566,14 @@ BOOKMARK_SAVE = _("Save bookmarks")
 # the feature without the need to get into a GUI.
 MOUSE_REVIEW_TOGGLE = _("Toggle mouse review mode")
 
+# Translators: Orca has a command to present the battery status (e.g. level, whether
+# or not it is plugged in, etc.). This string is the name of that command.
+PRESENT_BATTERY_STATUS = _("Present battery status")
+
+# Translators: Orca has a command to present the CPU and memory usage as percents.
+# This string is the name of that command.
+PRESENT_CPU_AND_MEMORY_USAGE = _("Present CPU and memory usage")
+
 # Translators: Orca has a command to present the current time in speech and in
 # braille.
 PRESENT_CURRENT_TIME = _("Present current time")
@@ -571,12 +587,14 @@ PRESENT_CURRENT_DATE = _("Present current date")
 # of keyboard shortcuts.
 PRESENT_SIZE_AND_POSITION = _("Present size and location of current object")
 
-# Translators: Orca normally intercepts all keyboard commands and only passes
-# them along to the current application when they are not Orca commands. This
-# command causes the next command issued to be passed along to the current
-# application, bypassing Orca's interception of it.
-BYPASS_NEXT_COMMAND = \
-    _("Pass the next command on to the current application")
+# Translators: This command toggles all (other) Orca commands so that the
+# associated keystroke can by consumed by the native application. For example,
+# if there were an Orca command bound to Alt+Down, normally pressing Alt+Down
+# would cause the Orca command to be used. This would mean Alt+Down could not
+# be used in editors to move the current line of text down. By temporarily
+# disabling Orca commands, Alt+Down would be ignored by Orca and work as
+# expected in the editor.
+BYPASS_MODE_TOGGLE = _("Toggle all Orca command keys")
 
 # Translators: Orca has a command to review previous chat room messages in
 # speech and braille. This string to be translated is associated with the
@@ -819,6 +837,11 @@ PRESENT_INPUT_LINE = _("Present the contents of the input line")
 # writing functions.
 STRUCTURAL_NAVIGATION_TOGGLE = _("Toggle structural navigation keys")
 
+# Translators: Orca has commands for navigating within a table, e.g. to the
+# next cell in a given direction. This string is the description of the command
+# which enables/disables this support.
+TABLE_NAVIGATION_TOGGLE = _("Toggle table navigation keys")
+
 # Translators: this is for navigating among blockquotes in a document.
 BLOCKQUOTE_PREV = _("Go to previous blockquote")
 
@@ -1084,6 +1107,18 @@ TABLE_CELL_RIGHT = _("Go right one cell")
 # Translators: this is for navigating among table cells in a document.
 TABLE_CELL_UP = _("Go up one cell")
 
+# Translators: this is for navigating among table cells in a document.
+TABLE_CELL_BEGINNING_OF_ROW = _("Go to the beginning of the row")
+
+# Translators: this is for navigating among table cells in a document.
+TABLE_CELL_END_OF_ROW = _("Go to the end of the row")
+
+# Translators: this is for navigating among table cells in a document.
+TABLE_CELL_TOP_OF_COLUMN = _("Go to the top of the column")
+
+# Translators: this is for navigating among table cells in a document.
+TABLE_CELL_BOTTOM_OF_COLUMN = _("Go to the bottom of the column")
+
 # Translators: When presenting the content of a line on a web page, Orca by
 # default presents the full line, including any links or form fields on that
 # line, in order to reflect the on-screen layout as seen by sighted users.
