@@ -19,6 +19,9 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
+# For the "AXUtilities has no ... member"
+# pylint: disable=E1101
+
 """Heuristic means to infer the functional/displayed label of a widget."""
 
 __id__        = "$Id$"
@@ -27,9 +30,13 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (C) 2011-2013 Igalia, S.L."
 __license__   = "LGPL"
 
-import pyatspi
-
 from . import debug
+from .ax_component import AXComponent
+from .ax_hypertext import AXHypertext
+from .ax_object import AXObject
+from .ax_table import AXTable
+from .ax_text import AXText
+from .ax_utilities import AXUtilities
 
 class LabelInference:
 
@@ -55,38 +62,41 @@ class LabelInference:
         Returns the text which we think is the label, or None.
         """
 
-        debug.println(debug.LEVEL_INFO, "INFER label for: %s" % obj, True)
+        tokens = ["LABEL INFERENCE: Infer label for", obj]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         if not obj:
             return None, []
 
-        if focusedOnly and not obj.getState().contains(pyatspi.STATE_FOCUSED):
-            debug.println(debug.LEVEL_INFO, "INFER - object not focused", True)
+        if focusedOnly and not AXUtilities.is_focused(obj):
+            tokens = ["LABEL INFERENCE:", obj, "is not focused"]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return None, []
 
         result, objects = None, []
         if not result:
             result, objects = self.inferFromTextLeft(obj)
-            debug.println(debug.LEVEL_INFO, "INFER - Text Left: %s" % result, True)
+            debug.printMessage(debug.LEVEL_INFO, f"LABEL INFERENCE: Text Left: '{result}'", True)
         if not result or self._preferRight(obj):
             result, objects = self.inferFromTextRight(obj) or result
-            debug.println(debug.LEVEL_INFO, "INFER - Text Right: %s" % result, True)
+            debug.printMessage(debug.LEVEL_INFO, f"LABEL INFERENCE: Text Right: '{result}'", True)
         if not result:
             result, objects = self.inferFromTable(obj)
-            debug.println(debug.LEVEL_INFO, "INFER - Table: %s" % result, True)
+            debug.printMessage(debug.LEVEL_INFO, f"LABEL INFERENCE: Table: '{result}'", True)
         if not result:
             result, objects = self.inferFromTextAbove(obj)
-            debug.println(debug.LEVEL_INFO, "INFER - Text Above: %s" % result, True)
+            debug.printMessage(debug.LEVEL_INFO, f"LABEL INFERENCE: Text Above: '{result}'", True)
         if not result:
             result, objects = self.inferFromTextBelow(obj)
-            debug.println(debug.LEVEL_INFO, "INFER - Text Below: %s" % result, True)
+            debug.printMessage(debug.LEVEL_INFO, f"LABEL INFERENCE: Text Below: '{result}'", True)
 
         # TODO - We probably do not wish to "infer" from these. Instead, we
         # should ensure that this content gets presented as part of the widget.
         # (i.e. the label is something on screen. Widget name and description
         # are each something other than a label.)
         if not result:
-            result, objects = obj.name, []
-            debug.println(debug.LEVEL_INFO, "INFER - Name: %s" % result, True)
+            result, objects = AXObject.get_name(obj), []
+            debug.printMessage(debug.LEVEL_INFO, f"LABEL INFERENCE: Name: '{result}'", True)
+
         if result:
             result = result.strip()
             result = result.replace("\n", " ")
@@ -94,7 +104,9 @@ class LabelInference:
         # Desperate times call for desperate measures....
         if not result:
             result, objects = self.inferFromTextLeft(obj, proximity=200)
-            debug.println(debug.LEVEL_INFO, "INFER - Text Left with proximity of 200: %s" % result, True)
+            debug.printMessage(
+                debug.LEVEL_INFO,
+                f"LABEL INFERENCE: Text Left with proximity of 200: '{result}'", True)
 
         self.clearCache()
         return result, objects
@@ -110,63 +122,43 @@ class LabelInference:
         """Returns True if we should prefer text on the right, rather than the
         left, for the object obj."""
 
-        onRightRoles = [pyatspi.ROLE_CHECK_BOX, pyatspi.ROLE_RADIO_BUTTON]
-        return obj.getRole() in onRightRoles
+        return AXUtilities.is_check_box(obj) or AXUtilities.is_radio_button(obj)
 
     def _preventRight(self, obj):
         """Returns True if we should not permit inference based on text to
         the right for the object obj."""
 
-        roles = [pyatspi.ROLE_COMBO_BOX,
-                 pyatspi.ROLE_LIST,
-                 pyatspi.ROLE_LIST_BOX]
-
-        return obj.getRole() in roles
+        return AXUtilities.is_combo_box(obj) or AXUtilities.is_list_box(obj)
 
     def _preferTop(self, obj):
         """Returns True if we should prefer text above, rather than below for
         the object obj."""
 
-        roles = [pyatspi.ROLE_COMBO_BOX,
-                 pyatspi.ROLE_LIST,
-                 pyatspi.ROLE_LIST_BOX]
-
-        return obj.getRole() in roles
+        return AXUtilities.is_combo_box(obj) or AXUtilities.is_list_box(obj)
 
     def _preventBelow(self, obj):
         """Returns True if we should not permit inference based on text below
         the object obj."""
 
-        roles = [pyatspi.ROLE_ENTRY,
-                 pyatspi.ROLE_PASSWORD_TEXT]
-
-        return obj.getRole() not in roles
+        return not AXUtilities.is_text_input(obj)
 
     def _isSimpleObject(self, obj):
         """Returns True if the given object has 'simple' contents, such as text
         without embedded objects or a single embedded object without text."""
 
-        if not obj:
+        if obj is None:
             return False
 
-        isMatch = lambda x: x and not self._script.utilities.isStaticTextLeaf(x)
+        def isMatch(x):
+            return x is not None \
+                  and not self._script.utilities.isStaticTextLeaf(x) \
+                  and not AXUtilities.is_link(x)
 
-        try:
-            children = [child for child in obj if isMatch(child)]
-        except (LookupError, RuntimeError):
-            debug.println(debug.LEVEL_INFO, 'Dead Accessible in %s' % obj, True)
-            return False
-
-        children = [x for x in children if x.getRole() != pyatspi.ROLE_LINK]
+        children = [child for child in AXObject.iter_children(obj, isMatch)]
         if len(children) > 1:
             return False
 
-        try:
-            text = obj.queryText()
-        except NotImplementedError:
-            return True
-
-        string = text.getText(0, -1).strip()
+        string = AXText.get_all_text(obj).strip()
         if string.count(self._script.EMBEDDED_OBJECT_CHARACTER) > 1:
             return False
 
@@ -175,11 +167,10 @@ class LabelInference:
     def _cannotLabel(self, obj):
         """Returns True if the given object should not be treated as a label."""
 
-        if not obj:
+        if obj is None:
             return True
 
-        nonLabelTextRoles = [pyatspi.ROLE_HEADING, pyatspi.ROLE_LIST_ITEM]
-        if obj.getRole() in nonLabelTextRoles:
+        if AXUtilities.is_heading(obj) or AXUtilities.is_list_item(obj):
             return True
 
         return self._isWidget(obj)
@@ -187,27 +178,15 @@ class LabelInference:
     def _isWidget(self, obj):
         """Returns True if the given object is a widget."""
 
-        if not obj:
+        if obj is None:
             return False
 
         rv = self._isWidgetCache.get(hash(obj))
         if rv is not None:
             return rv
 
-        widgetRoles = [pyatspi.ROLE_CHECK_BOX,
-                       pyatspi.ROLE_RADIO_BUTTON,
-                       pyatspi.ROLE_TOGGLE_BUTTON,
-                       pyatspi.ROLE_COMBO_BOX,
-                       pyatspi.ROLE_LIST,
-                       pyatspi.ROLE_LIST_BOX,
-                       pyatspi.ROLE_MENU,
-                       pyatspi.ROLE_MENU_ITEM,
-                       pyatspi.ROLE_ENTRY,
-                       pyatspi.ROLE_PASSWORD_TEXT,
-                       pyatspi.ROLE_PUSH_BUTTON]
-
-        isWidget = obj.getRole() in widgetRoles
-        if not isWidget and obj.getState().contains(pyatspi.STATE_EDITABLE):
+        isWidget = AXUtilities.is_widget(obj) or AXUtilities.is_menu_related(obj)
+        if not isWidget and AXUtilities.is_editable(obj):
             isWidget = True
 
         self._isWidgetCache[hash(obj)] = isWidget
@@ -226,31 +205,16 @@ class LabelInference:
             return rv
 
         extents = 0, 0, 0, 0
-        text = self._script.utilities.queryNonEmptyText(obj)
-        if text:
-            skipTextExtents = [pyatspi.ROLE_ENTRY, pyatspi.ROLE_PASSWORD_TEXT]
-            if not obj.getRole() in skipTextExtents:
+        if AXObject.supports_text(obj):
+            if not AXUtilities.is_text_input(obj):
                 if endOffset == -1:
-                    try:
-                        endOffset = text.characterCount
-                    except:
-                        msg = "ERROR: Exception getting character count for %s" % obj
-                        debug.println(debug.LEVEL_INFO, msg, True)
-                        return extents
-
-                extents = text.getRangeExtents(startOffset, endOffset, 0)
+                    endOffset = AXText.get_character_count(obj)
+                rect = AXText.get_range_rect(obj, startOffset, endOffset)
+                extents = rect.x, rect.y, rect.width, rect.height
 
         if not (extents[2] and extents[3]):
-            try:
-                ext = obj.queryComponent().getExtents(0)
-            except NotImplementedError:
-                msg = "INFO: %s does not implement the component interface" % obj
-                debug.println(debug.LEVEL_INFO, msg, True)
-            except:
-                msg = "ERROR: Exception getting extents for %s" % obj
-                debug.println(debug.LEVEL_INFO, msg, True)
-            else:
-                extents = ext.x, ext.y, ext.width, ext.height
+            ext = AXComponent.get_rect(obj)
+            extents = ext.x, ext.y, ext.width, ext.height
 
         self._extentsCache[(hash(obj), startOffset, endOffset)] = extents
         return extents
@@ -282,8 +246,8 @@ class LabelInference:
 
         key = hash(obj)
         if self._isWidget(obj):
-            start, end = self._script.utilities.getHyperlinkRange(obj)
-            obj = obj.parent
+            start = AXHypertext.get_link_start_offset(obj)
+            obj = AXObject.get_parent(obj)
 
         rv = self._script.utilities.getLineContentsAtOffset(obj, start, True, False)
         self._lineCache[key] = rv
@@ -465,53 +429,31 @@ class LabelInference:
         return None, []
 
     def _isTable(self, obj):
-        if not obj:
-            return False
-
-        if obj.getRole() == pyatspi.ROLE_TABLE:
+        if AXUtilities.is_table(obj):
             return True
 
         return self._getTag(obj) == 'table'
 
     def _isRow(self, obj):
-        if not obj:
-            return False
-
-        if obj.getRole() == pyatspi.ROLE_TABLE_ROW:
+        if AXUtilities.is_table_row(obj):
             return True
 
         return self._getTag(obj) == 'tr'
 
     def _isCell(self, obj):
-        if not obj:
-            return False
-
-        if obj.getRole() == pyatspi.ROLE_TABLE_CELL:
+        if AXUtilities.is_table_cell(obj):
             return True
 
         return self._getTag(obj) in ['td', 'th']
 
-    def _getCellFromTable(self, table, rowindex, colindex):
-        if "Table" not in pyatspi.listInterfaces(table):
-            return None
-
-        if rowindex < 0 or colindex < 0:
-            return None
-
-        iface = table.queryTable()
-        if rowindex >= iface.nRows or colindex >= iface.nColumns:
-            return None
-
-        return table.queryTable().getAccessibleAt(rowindex, colindex)
-
     def _getCellFromRow(self, row, colindex):
-        if 0 <= colindex < row.childCount:
+        if 0 <= colindex < AXObject.get_child_count(row):
             return row[colindex]
 
         return None
 
     def _getTag(self, obj):
-        attrs = self._script.utilities.objectAttributes(obj)
+        attrs = AXObject.get_attributes_dict(obj)
         return attrs.get('tag')
 
     def inferFromTable(self, obj, proximityForRight=50):
@@ -526,34 +468,38 @@ class LabelInference:
         Returns the text which we think is the label, or None.
         """
 
-        cell = pyatspi.findAncestor(obj, self._isCell)
+        cell = AXObject.find_ancestor(obj, self._isCell)
         if not self._isSimpleObject(cell):
             return None, []
 
-        if not cell in [obj.parent, obj.parent.parent]:
+        parent = AXObject.get_parent(obj)
+        if cell not in [parent, AXObject.get_parent(parent)]:
             return None, []
 
-        grid = pyatspi.findAncestor(cell, self._isTable)
+        grid = AXObject.find_ancestor(cell, self._isTable)
         if not grid:
             return None, []
 
         cellLeft = cellRight = cellAbove = cellBelow = None
-        gridrow = pyatspi.findAncestor(cell, self._isRow)
-        rowindex, colindex = self._script.utilities.coordinatesForCell(cell)
+        gridrow = AXObject.find_ancestor(cell, self._isRow)
+        rowindex, colindex = AXTable.get_cell_coordinates(cell, prefer_attribute=False)
         if colindex > -1:
-            cellLeft = self._getCellFromTable(grid, rowindex, colindex - 1)
-            cellRight = self._getCellFromTable(grid, rowindex, colindex + 1)
-            cellAbove = self._getCellFromTable(grid, rowindex - 1, colindex)
-            cellBelow = self._getCellFromTable(grid, rowindex + 1, colindex)
-        elif gridrow and cell.parent == gridrow:
-            cellindex = cell.getIndexInParent()
+            cellLeft = AXTable.get_cell_at(grid, rowindex, colindex - 1)
+            cellRight = AXTable.get_cell_at(grid, rowindex, colindex + 1)
+            cellAbove = AXTable.get_cell_at(grid, rowindex - 1, colindex)
+            cellBelow = AXTable.get_cell_at(grid, rowindex + 1, colindex)
+        elif gridrow and AXObject.get_parent(cell) == gridrow:
+            cellindex = AXObject.get_index_in_parent(cell)
             cellLeft = self._getCellFromRow(gridrow, cellindex - 1)
             cellRight = self._getCellFromRow(gridrow, cellindex + 1)
-            rowindex = gridrow.getIndexInParent()
+            rowindex = AXObject.get_index_in_parent(gridrow)
+            gridrowParent = AXObject.get_parent(gridrow)
             if rowindex > 0:
-                cellAbove = self._getCellFromRow(gridrow.parent[rowindex - 1], cellindex)
-            if rowindex + 1 < grid.childCount:
-                cellBelow = self._getCellFromRow(gridrow.parent[rowindex + 1], cellindex)
+                rowAbove = AXObject.get_child(gridrowParent, rowindex - 1)
+                cellAbove = self._getCellFromRow(rowAbove, cellindex)
+            if rowindex + 1 < AXObject.get_child_count(grid):
+                rowBelow = AXObject.get_child(gridrowParent, rowindex + 1)
+                cellBelow = self._getCellFromRow(rowBelow, cellindex)
 
         if cellLeft and not self._preferRight(obj):
             label, sources = self._createLabelFromContents(cellLeft)
@@ -597,21 +543,22 @@ class LabelInference:
         # as a functional label. Therefore, see if this table looks like a grid
         # of widgets with the functional labels in the first row.
 
-        try:
-            table = grid.queryTable()
-        except NotImplementedError:
-            return None, []
-
-        firstRow = [table.getAccessibleAt(0, i) for i in range(table.nColumns)]
+        columns = AXTable.get_column_count(grid)
+        firstRow = [AXTable.get_cell_at(grid, 0, i) for i in range(columns)]
         if not firstRow or list(filter(self._isWidget, firstRow)):
             return None, []
 
         if colindex < 0:
             return None, []
 
-        cells = [table.getAccessibleAt(i, colindex) for i in range(1, table.nRows)]
-        cells = [x for x in cells if x is not None]
-        if [x for x in cells if x.childCount and x[0].getRole() != obj.getRole()]:
+        def isMatch(x):
+            if not AXObject.get_child_count(x):
+                return False
+            return not AXUtilities.have_same_role(AXObject.get_child(x, 0), obj)
+
+        rows = AXTable.get_row_count(grid)
+        cells = [AXTable.get_cell_at(grid, i, colindex) for i in range(1, rows)]
+        if list(filter(isMatch, cells)):
             return None, []
 
         label, sources = self._createLabelFromContents(firstRow[colindex])
