@@ -1,15 +1,10 @@
-__all__ = ['Banshee',
-           'Eclipse',
-           'evince',
+__all__ = ['evince',
            'evolution',
            'gajim',
-           'gcalctool',
            'gedit',
-           'gnome-documents',
            'gnome-shell',
            'kwin',
            'notification-daemon',
-           'notify-osd',
            'pidgin',
            'soffice',
            'SeaMonkey',
