@@ -26,26 +26,18 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2010-2013 Igalia, S.L."
 __license__   = "LGPL"
 
-import orca.debug as debug
-import orca.orca as orca
-import orca.orca_state as orca_state
-import orca.scripts.toolkits.clutter as clutter
+from orca import debug
+from orca import focus_manager
+from orca.scripts.toolkits import clutter
 from orca.ax_object import AXObject
 from orca.ax_utilities import AXUtilities
 
-from .formatting import Formatting
 from .script_utilities import Utilities
 
+
 class Script(clutter.Script):
 
-    def __init__(self, app):
-        clutter.Script.__init__(self, app)
-
-    def getFormatting(self):
-        """Returns the formatting strings for this script."""
-        return Formatting(self)
-
-    def getUtilities(self):
+    def get_utilities(self):
         return Utilities(self)
 
     def deactivate(self):
@@ -54,61 +46,49 @@ class Script(clutter.Script):
         self.utilities.clearCachedObjects()
         super().deactivate()
 
-    def skipObjectEvent(self, event):
-        """Determines whether or not this event should be skipped due to
-        being redundant, part of an event flood, etc."""
+    def locus_of_focus_changed(self, event, old_focus, new_focus):
+        """Handles changes of focus of interest to the script."""
 
-        if AXUtilities.is_window(event.source):
-            return self.utilities.isBogusWindowFocusClaim(event)
-
-        return clutter.Script.skipObjectEvent(self, event)
-
-    def locusOfFocusChanged(self, event, oldFocus, newFocus):
         if event is not None and event.type == "window:activate" \
-          and newFocus is not None and not AXObject.get_name(newFocus):
-            queuedEvent = self._getQueuedEvent("object:state-changed:focused", True)
+          and new_focus is not None and not AXObject.get_name(new_focus):
+            queuedEvent = self._get_queued_event("object:state-changed:focused", True)
             if queuedEvent and queuedEvent.source != event.source:
                 msg = "GNOME SHELL: Have matching focused event. Not announcing nameless window."
                 debug.printMessage(debug.LEVEL_INFO, msg, True)
                 return
 
-        super().locusOfFocusChanged(event, oldFocus, newFocus)
+        super().locus_of_focus_changed(event, old_focus, new_focus)
 
-    def onNameChanged(self, event):
+    def on_name_changed(self, event):
         """Callback for object:property-change:accessible-name events."""
 
         if not AXUtilities.is_label(event.source):
-            clutter.Script.onNameChanged(self, event)
+            clutter.Script.on_name_changed(self, event)
             return
 
         # If we're already in a dialog, and a label inside that dialog changes its name,
         # present the new name. Example: the "Command not found" label in the Run dialog.
-        dialog = AXObject.find_ancestor(orca_state.locusOfFocus, AXUtilities.is_dialog)
-        tokens = ["GNOME SHELL: focus", orca_state.locusOfFocus, "is in dialog:", dialog]
+        dialog = AXObject.find_ancestor(
+            focus_manager.get_manager().get_locus_of_focus(), AXUtilities.is_dialog)
+        tokens = ["GNOME SHELL: focus is in dialog:", dialog]
         debug.printTokens(debug.LEVEL_INFO, tokens, True)
         if dialog and AXObject.is_ancestor(event.source, dialog):
             msg = "GNOME SHELL: Label changed name in current dialog. Presenting."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             self.presentMessage(AXObject.get_name(event.source))
 
-    def onSelectedChanged(self, event):
+    def on_selected_changed(self, event):
         """Callback for object:state-changed:selected accessibility events."""
 
-        # Some buttons, like the Wikipedia button, claim to be selected but
-        # lack STATE_SELECTED. The other buttons, such as in the Dash and
-        # event switcher, seem to have the right state. Since the ones with
-        # the wrong state seem to be things we don't want to present anyway
-        # we'll stop doing so and hope we are right.
-        if event.detail1:
-            if AXUtilities.is_panel(event.source):
-                AXObject.clear_cache(event.source)
-            if AXUtilities.is_selected(event.source):
-                orca.setLocusOfFocus(event, event.source)
+        # gnome-shell fails to implement the selection interface but fires state-changed
+        # selected in the switcher and similar containers.
+        if AXUtilities.is_selected(event.source):
+            focus_manager.get_manager().set_locus_of_focus(event, event.source)
             return
 
-        clutter.Script.onSelectedChanged(self, event)
+        clutter.Script.on_selected_changed(self, event)
 
-    def onFocusedChanged(self, event):
+    def on_focused_changed(self, event):
         """Callback for object:state-changed:focused accessibility events."""
 
         if not event.detail1:
@@ -120,36 +100,18 @@ class Script(clutter.Script):
             return
 
         if not AXObject.get_name(event.source) and AXUtilities.is_menu_item(event.source) \
-           and not self.utilities.labelsForObject(event.source):
+           and not AXUtilities.get_is_labelled_by(event.source):
             descendant = AXObject.find_descendant(event.source, AXUtilities.is_slider)
             if descendant is not None:
-                orca.setLocusOfFocus(event, descendant)
+                focus_manager.get_manager().set_locus_of_focus(event, descendant)
                 return
 
-        clutter.Script.onFocusedChanged(self, event)
+        clutter.Script.on_focused_changed(self, event)
 
-    def echoPreviousWord(self, obj, offset=None):
-        try:
-            text = obj.queryText()
-        except NotImplementedError:
-            return False
+    def is_activatable_event(self, event):
+        """Returns True if event should cause this script to become active."""
 
-        if not offset:
-            if text.caretOffset == -1:
-                offset = text.characterCount - 1
-            else:
-                offset = text.caretOffset - 1
-
-        if offset == 0:
-            return False
-
-        return super().echoPreviousWord(obj, offset)
-
-    def isActivatableEvent(self, event):
         if event.type.startswith('object:state-changed:selected') and event.detail1:
             return True
 
-        if self.utilities.isBogusWindowFocusClaim(event):
-            return False
-
-        return super().isActivatableEvent(event)
+        return super().is_activatable_event(event)
