@@ -34,12 +34,7 @@ from .script_utilities import Utilities
 
 class Script(switcher.Script, Qt.Script):
 
-    def __init__(self, app):
-        """Creates a new script for the given application."""
-
-        super().__init__(app)
-
-    def getUtilities(self):
+    def get_utilities(self):
         """Returns the utilities for this script."""
 
         return Utilities(self)
