@@ -17,7 +17,9 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
-""" Custom script for The notification daemon."""
+""" Custom script for notification daemon."""
 
-from script import Script
+# https://gitlab.gnome.org/GNOME/orca/-/issues/358
+# ruff: noqa: F401
+from .script import Script
 
