@@ -31,6 +31,7 @@ from orca import debug
 from orca import messages
 from orca import spellcheck
 from orca.ax_object import AXObject
+from orca.ax_text import AXText
 from orca.ax_utilities import AXUtilities
 
 class SpellCheck(spellcheck.SpellCheck):
@@ -68,6 +69,16 @@ class SpellCheck(spellcheck.SpellCheck):
             debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return False
 
+        # for LO >= 25.2, dialog has an accessible ID of "SpellingDialog" set
+        # (older versions have no ID set, will fall back to heuristics further below)
+        dialog_id = dialog.get_accessible_id()
+        if dialog_id:
+            rv = (dialog_id == "SpellingDialog")
+            self._windows[hash(dialog)] = rv
+            tokens = ["SOFFICE:", dialog, "is spellcheck dialog based on accessible ID:", rv]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            return rv
+
         if AXObject.find_descendant(dialog, AXUtilities.is_page_tab_list) is not None:
             self._windows[hash(window)] = False
             self._windows[hash(dialog)] = False
@@ -107,21 +118,17 @@ class SpellCheck(spellcheck.SpellCheck):
         return rv
 
     def _getSuggestionIndexAndPosition(self, suggestion):
-        index, total = self._script.utilities.getPositionAndSetSize(suggestion)
+        index = AXUtilities.get_position_in_set(suggestion)
+        total = AXUtilities.get_set_size(suggestion)
         return index + 1, total
 
     def getMisspelledWord(self):
-        try:
-            text = self._errorWidget.queryText()
-        except Exception:
-            return ""
-
+        length = AXText.get_character_count(self._errorWidget)
         offset, string = 0, ""
-        while 0 <= offset < text.characterCount:
-            attributes, start, end = text.getAttributeRun(offset, False)
-            attrs = dict([attr.split(":", 1) for attr in attributes])
+        while 0 <= offset < length:
+            attrs, start, end = AXText.get_text_attributes_at_offset(self._errorWidget, offset)
             if attrs.get("fg-color", "").replace(" ", "") == "255,0,0":
-                return text.getText(start, end)
+                return AXText.get_substring(self._errorWidget, start, end)
             offset = max(end, offset + 1)
 
         return string
@@ -130,16 +137,11 @@ class SpellCheck(spellcheck.SpellCheck):
         if not self.isActive():
             return False
 
-        try:
-            text = self._errorWidget.queryText()
-        except Exception:
-            return False
-
-        string = text.getText(0, -1)
+        string = AXText.get_all_text(self._errorWidget)
         if not string:
             return False
 
         msg = messages.MISSPELLED_WORD_CONTEXT % string
-        voice = self._script.speechGenerator.voice(string=msg)
+        voice = self._script.speech_generator.voice(string=msg)
         self._script.speakMessage(msg, voice=voice)
         return True
