@@ -18,6 +18,8 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
+"""Produces braille presentation for accessible objects."""
+
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
@@ -25,14 +27,25 @@ __copyright__ = "Copyright (c) 2016 Igalia, S.L."
 __license__   = "LGPL"
 
 from orca import braille_generator
+from orca import debug
 
 
 class BrailleGenerator(braille_generator.BrailleGenerator):
+    """Produces braille presentation for accessible objects."""
 
-    def __init__(self, script):
-        super().__init__(script)
+    @staticmethod
+    def log_generator_output(func):
+        """Decorator for logging."""
 
-    def _generateDescription(self, obj, **args):
+        def wrapper(*args, **kwargs):
+            result = func(*args, **kwargs)
+            tokens = [f"TERMINAL BRAILLE GENERATOR: {func.__name__}:", result]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            return result
+        return wrapper
+
+    @log_generator_output
+    def _generate_accessible_description(self, _obj, **_args):
         # The text in the description is the same as the text in the page
         # tab and similar to (and sometimes the same as) the prompt.
         return []
