@@ -25,7 +25,6 @@ __copyright__ = "Copyright (c) 2016 Igalia, S.L."
 __license__   = "LGPL"
 
 from orca import debug
-from orca import focus_manager
 from orca.scripts import default
 from orca.ax_text import AXText
 
@@ -38,7 +37,7 @@ class Script(default.Script):
 
     def __init__(self, app):
         super().__init__(app)
-        self.presentIfInactive = False
+        self.present_if_inactive = False
 
     def deactivate(self):
         """Called when this script is deactivated."""
@@ -46,28 +45,22 @@ class Script(default.Script):
         self.utilities.clearCache()
         super().deactivate()
 
-    def getBrailleGenerator(self):
+    def get_braille_generator(self):
         """Returns the braille generator for this script."""
 
         return BrailleGenerator(self)
 
-    def getSpeechGenerator(self):
+    def get_speech_generator(self):
         """Returns the speech generator for this script."""
 
         return SpeechGenerator(self)
 
-    def getUtilities(self):
+    def get_utilities(self):
         """Returns the utilities for this script."""
 
         return Utilities(self)
 
-    def onFocus(self, event):
-        """Callback for focus: accessibility events."""
-
-        # https://bugzilla.gnome.org/show_bug.cgi?id=748311
-        focus_manager.getManager().set_locus_of_focus(event, event.source)
-
-    def onTextDeleted(self, event):
+    def on_text_deleted(self, event):
         """Callback for object:text-changed:delete accessibility events."""
 
         if self.utilities.treatEventAsNoise(event):
@@ -75,30 +68,30 @@ class Script(default.Script):
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
-        super().onTextDeleted(event)
+        super().on_text_deleted(event)
 
-    def onTextInserted(self, event):
+    def on_text_inserted(self, event):
         """Callback for object:text-changed:insert accessibility events."""
 
         if not self.utilities.treatEventAsCommand(event):
             msg = "TERMINAL: Passing along event to default script."
             debug.printMessage(debug.LEVEL_INFO, msg, True)
-            super().onTextInserted(event)
+            super().on_text_inserted(event)
             return
 
         msg = "TERMINAL: Insertion is believed to be due to terminal command"
         debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-        self.updateBraille(event.source)
+        self.update_braille(event.source)
 
         newString = self.utilities.insertedText(event)
         if len(newString) == 1:
-            self.speakCharacter(newString)
+            self.speak_character(newString)
         else:
-            voice = self.speechGenerator.voice(obj=event.source, string=newString)
+            voice = self.speech_generator.voice(obj=event.source, string=newString)
             self.speakMessage(newString, voice=voice)
 
-        if self.flatReviewPresenter.is_active():
+        if self.get_flat_review_presenter().is_active():
             msg = "TERMINAL: Flat review presenter is active. Ignoring insertion"
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
@@ -108,48 +101,26 @@ class Script(default.Script):
         self.utilities.updateCachedTextSelection(event.source)
 
     def presentKeyboardEvent(self, event):
-        if not event.isPrintableKey():
+        if not event.is_printable_key():
             return super().presentKeyboardEvent(event)
 
-        if event.isPressedKey():
+        if event.is_pressed_key():
             return False
 
         self._sayAllIsInterrupted = False
         self.utilities.clearCachedCommandState()
-        if not event.shouldEcho or event.isOrcaModified() or event.isCharacterEchoable():
+        if not event.should_echo() or event.is_orca_modified() or event.is_character_echoable():
             return False
 
         # We have no reliable way of knowing a password is being entered into
         # a terminal -- other than the fact that the text typed isn't there.
-        char, start = AXText.get_character_at_offset(event.getObject())[0:2]
-        prevChar = AXText.get_character_at_offset(event.getObject(), start - 1)[0]
+        char, start = AXText.get_character_at_offset(event.get_object())[0:2]
+        prevChar = AXText.get_character_at_offset(event.get_object(), start - 1)[0]
         string = event.event_string
         if string not in [prevChar, "space", char]:
             return False
 
         tokens = ["TERMINAL: Presenting keyboard event", string]
         debug.printTokens(debug.LEVEL_INFO, tokens, True)
-        self.speakKeyEvent(event)
-        return True
-
-    def skipObjectEvent(self, event):
-        if event.type == "object:text-changed:insert":
-            return False
-
-        newEvent, newTime = None, 0
-        if event.type == "object:text-changed:delete":
-            if self.utilities.isBackSpaceCommandTextDeletionEvent(event):
-                return False
-
-            newEvent, newTime = self.eventCache.get("object:text-changed:insert", [None, 0])
-
-        if newEvent is None or newEvent.source != event.source:
-            return super().skipObjectEvent(event)
-
-        if event.detail1 != newEvent.detail1:
-            return False
-
-        data = "\n%s%s" % (" " * 11, str(newEvent).replace("\t", " " * 11))
-        tokens = ["TERMINAL: Skipping due to more recent event at offset", data]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        self.speak_key_event(event)
         return True
