@@ -19,6 +19,9 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
+# For the "AXUtilities has no ... member"
+# pylint: disable=E1101
+
 """Custom script utilities for Chromium"""
 
 __id__        = "$Id$"
@@ -27,15 +30,14 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2018-2019 Igalia, S.L."
 __license__   = "LGPL"
 
-import gi
-gi.require_version("Atspi", "2.0")
-from gi.repository import Atspi
 import re
 
 from orca import debug
-from orca import orca_state
+from orca import focus_manager
+from orca import input_event_manager
 from orca.scripts import web
 from orca.ax_object import AXObject
+from orca.ax_text import AXText
 from orca.ax_utilities import AXUtilities
 
 
@@ -69,8 +71,8 @@ class Utilities(web.Utilities):
         if rv is not None:
             return rv
 
-        roles = [Atspi.Role.STATIC, Atspi.Role.TEXT]
-        rv = AXObject.get_role(obj) in roles and self._getTag(obj) in (None, "", "br")
+        rv = AXUtilities.is_static(obj) or AXUtilities.is_text(obj) \
+            and self._getTag(obj) in (None, "", "br")
         if rv:
             tokens = ["CHROMIUM:", obj, "believed to be static text leaf"]
             debug.printTokens(debug.LEVEL_INFO, tokens, True)
@@ -113,7 +115,7 @@ class Utilities(web.Utilities):
             elif AXObject.get_child_count(parent) > 1:
                 rv = AXObject.get_child(parent, 0) == obj
             else:
-                rv = AXObject.get_name(obj) != self.displayedText(parent)
+                rv = AXObject.get_name(obj) != AXText.get_all_text(parent)
 
         self._isListItemMarker[hash(obj)] = rv
         return rv
@@ -139,7 +141,8 @@ class Utilities(web.Utilities):
         # is opened/expanded, a menu with that same name appears. It would be
         # nice if there were a connection (parent/child or an accessible relation)
         # between the two....
-        return self.treatAsMenu(orca_state.locusOfFocus) and super().isPopupMenuForCurrentItem(obj)
+        return self.treatAsMenu(focus_manager.get_manager().get_locus_of_focus()) \
+            and super().isPopupMenuForCurrentItem(obj)
 
     def isFrameForPopupMenu(self, obj):
         # The ancestry of a popup menu appears to be a menu bar (even though
@@ -216,11 +219,11 @@ class Utilities(web.Utilities):
         return result
 
     def autocompleteForPopup(self, obj):
-        relation = AXObject.get_relation(obj, Atspi.RelationType.POPUP_FOR)
-        if not relation:
+        targets = AXUtilities.get_is_popup_for(obj)
+        if not targets:
             return None
 
-        target = relation.get_target(0)
+        target = targets[0]
         if AXUtilities.is_autocomplete(target):
             return target
 
@@ -237,40 +240,19 @@ class Utilities(web.Utilities):
             return False
 
         if event.type.startswith("object:text-caret-moved"):
-            lastKey, mods = self.lastKeyAndModifiers()
-            if lastKey in ["Down", "Up"]:
-                return True
+            return input_event_manager.get_manager().last_event_was_up_or_down()
 
         return False
 
     def setCaretPosition(self, obj, offset, documentFrame=None):
         super().setCaretPosition(obj, offset, documentFrame)
 
+        # TODO - JD: Is this hack still needed?
         link = AXObject.find_ancestor(obj, AXUtilities.is_link)
         if link is not None:
             tokens = ["CHROMIUM: HACK: Grabbing focus on", obj, "'s ancestor", link]
             debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            self.grabFocus(link)
-
-    def handleAsLiveRegion(self, event):
-        if not super().handleAsLiveRegion(event):
-            return False
-
-        if not event.type.startswith("object:children-changed:add"):
-            return True
-
-        # At least some of the time, we're getting text insertion events immediately
-        # followed by children-changed events to tell us that the object whose text
-        # changed is now being added to the accessibility tree. Furthermore the
-        # additions are not always coming to us in presentational order, whereas
-        # the text changes appear to be. So most of the time, we can ignore the
-        # children-changed events. Except for when we can't.
-        if AXUtilities.is_table(event.any_data):
-            return True
-
-        msg = "CHROMIUM: Event is believed to be redundant live region notification"
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        return False
+            AXObject.grab_focus(link)
 
     def getFindResultsCount(self, root=None):
         root = root or self._findContainer
@@ -282,7 +264,8 @@ class Utilities(web.Utilities):
             return ""
 
         bar = statusBars[0]
-        AXObject.clear_cache(bar)
+        # TODO - JD: Is this still needed?
+        AXObject.clear_cache(bar, False, "Ensuring we have correct name for find results.")
         if len(re.findall(r"\d+", AXObject.get_name(bar))) == 2:
             return AXObject.get_name(bar)
 
@@ -333,7 +316,7 @@ class Utilities(web.Utilities):
         return True
 
     def inFindContainer(self, obj=None):
-        obj = obj or orca_state.locusOfFocus
+        obj = obj or focus_manager.get_manager().get_locus_of_focus()
         if not (AXUtilities.is_entry(obj) or AXUtilities.is_push_button(obj)):
             return False
         if self.inDocumentContent(obj):
@@ -361,74 +344,9 @@ class Utilities(web.Utilities):
 
         # Don't bother if the root is a 'pre' or 'code' element. Those often have
         # nothing but a TON of static text leaf nodes, which we want to ignore.
-        if self._getTag(root) in ('pre', 'code'):
+        if AXUtilities.is_code(root):
             tokens = ["CHROMIUM: Returning 0 descendants for pre/code", root]
             debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return []
 
         return super().findAllDescendants(root, includeIf, excludeIf)
-
-    def accessibleAtPoint(self, root, x, y, coordType=None):
-        result = super().accessibleAtPoint(root, x, y, coordType)
-
-        # Chromium cannot do a hit test of web content synchronously. So what it
-        # does is return a guess, then fire off an async hit test. The next time
-        # one calls it, Chromium returns the previous async hit test result if
-        # the point is still within its bounds. Therefore, we need to call
-        # accessibleAtPoint() twice to be safe.
-        msg = "CHROMIUM: Getting accessibleAtPoint again due to async hit test result."
-        debug.printMessage(debug.LEVEL_INFO, msg, True)
-        result = super().accessibleAtPoint(root, x, y, coordType)
-        return result
-
-    def _isActiveAndShowingAndNotIconified(self, obj):
-        if super()._isActiveAndShowingAndNotIconified(obj):
-            return True
-
-        if obj and AXObject.get_application(obj) != self._script.app:
-            return False
-
-        # FIXME: This can potentially be non-performant because AT-SPI2 will recursively
-        # clear the cache of all descendants. This is an attempt to work around what may
-        # be a lack of window:activate and object:state-changed events from Chromium
-        # windows in at least some environments.
-        try:
-            tokens = ["CHROMIUM: Clearing cache for", obj]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            AXObject.clear_cache(obj)
-        except Exception:
-            tokens = ["CHROMIUM: Exception clearing cache for", obj]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            return False
-
-        if super()._isActiveAndShowingAndNotIconified(obj):
-            tokens = ["CHROMIUM:", obj, "deemed to be active and showing after cache clear"]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            return True
-
-        return False
-
-    def _shouldCalculatePositionAndSetSize(self, obj):
-        # Chromium calculates posinset and setsize for description lists based on the
-        # number of terms present. If we want to present the number of values associated
-        # with a given term, we need to work those values out ourselves.
-        if self.isDescriptionListDescription(obj):
-            return True
-
-        if self.inDocumentContent(obj):
-            return super()._shouldCalculatePositionAndSetSize(obj)
-
-        # Chromium has accessible menu items which are not focusable and therefore do not
-        # have a posinset and setsize calculated. But they may claim to be the selected
-        # item when an accessible child is selected (e.g. "zoom" when "+" or "-" gains focus.
-        # Normally we calculate posinset and setsize when the application hasn't provided it.
-        # We don't want to do that in the case of menu items like "zoom" because our result
-        # will not jibe with the values of its siblings. Thus if a sibling has a value,
-        # assume that the missing attributes are missing on purpose.
-        for sibling in AXObject.iter_children(AXObject.get_parent(obj)):
-            if self.getPositionInSet(sibling) is not None:
-                tokens = ["CHROMIUM:", obj, "'s sibling", sibling, "has posinset."]
-                debug.printTokens(debug.LEVEL_INFO, tokens, True)
-                return False
-
-        return True
