@@ -32,56 +32,19 @@ __license__   = "LGPL"
 
 import gi
 gi.require_version("Atspi", "2.0")
-from gi.repository import Atspi
 
 import re
-import time
 
 from orca import debug
-from orca import orca_state
+from orca import focus_manager
 from orca.scripts import web
+from orca.ax_document import AXDocument
 from orca.ax_object import AXObject
 from orca.ax_utilities import AXUtilities
 
 
 class Utilities(web.Utilities):
 
-    def __init__(self, script):
-        super().__init__(script)
-        self._lastAutoTextObjectEvent = None
-        self._lastAutoTextInputEvent = None
-        self._lastAutoTextEventTime = 0
-
-    def _attemptBrokenTextRecovery(self, obj, **args):
-        boundary = args.get('boundary')
-
-        # Gecko fails to implement this boundary type.
-        if boundary == Atspi.TextBoundaryType.SENTENCE_START:
-            return True
-
-        if self.isContentEditableWithEmbeddedObjects(obj):
-            return boundary == Atspi.TextBoundaryType.WORD_START
-
-        return True
-
-    def _treatAsLeafNode(self, obj):
-        if AXUtilities.is_table_row(obj):
-            return not AXObject.get_child_count(obj)
-
-        return super()._treatAsLeafNode(obj)
-
-    def containsPoint(self, obj, x, y, coordType, margin=2):
-        if not super().containsPoint(obj, x, y, coordType, margin):
-            return False
-
-        if (AXUtilities.is_menu(obj) or AXUtilities.is_tool_tip(obj)) \
-           and self.topLevelObject(obj) == AXObject.get_parent(obj):
-            tokens = ["GECKO:", obj, "is suspected to be off screen object"]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            return False
-
-        return True
-
     def isLayoutOnly(self, obj):
         if super().isLayoutOnly(obj):
             return True
@@ -91,33 +54,6 @@ class Utilities(web.Utilities):
 
         return False
 
-    def isSameObject(self, obj1, obj2, comparePaths=False, ignoreNames=False,
-                     ignoreDescriptions=True):
-        if super().isSameObject(obj1, obj2, comparePaths, ignoreNames, ignoreDescriptions):
-            return True
-
-        roles = self._topLevelRoles()
-        if not (AXObject.get_role(obj1) in roles and AXObject.get_role(obj2) in roles):
-            return False
-
-        rv = AXObject.get_name(obj1) == AXObject.get_name(obj2)
-        tokens = ["GECKO: Treating", obj1, "and", obj2, "as same object:", rv]
-        debug.printTokens(debug.LEVEL_INFO, tokens, True)
-        return rv
-
-    def isOnScreen(self, obj, boundingbox=None):
-        if not super().isOnScreen(obj, boundingbox):
-            return False
-        if not AXUtilities.is_unknown(obj):
-            return True
-
-        if self.topLevelObject(obj) == AXObject.get_parent(obj):
-            tokens = ["INFO:", obj, "is suspected to be off screen object"]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            return False
-
-        return True
-
     def getOnScreenObjects(self, root, extents=None):
         objects = super().getOnScreenObjects(root, extents)
 
@@ -156,7 +92,7 @@ class Utilities(web.Utilities):
         if "self-repair.mozilla.org" in name:
             return True
 
-        uri = self.documentFrameURI(obj)
+        uri = AXDocument.get_uri(obj)
         if uri.startswith("moz-extension"):
             return True
 
@@ -165,19 +101,6 @@ class Utilities(web.Utilities):
 
         return False
 
-    def _objectMightBeBogus(self, obj):
-        if AXUtilities.is_section(obj) and AXUtilities.is_frame(AXObject.get_parent(obj)):
-            tokens = ["GECKO:", obj, "is believed to be a bogus object"]
-            debug.printTokens(debug.LEVEL_INFO, tokens, True)
-            return True
-
-        return False
-
-    def canBeActiveWindow(self, window, clearCache=False):
-        # We apparently having missing events from Gecko requiring
-        # we update the cache. This is not performant. :(
-        return super().canBeActiveWindow(window, True)
-
     def treatAsEntry(self, obj):
         if not obj or self.inDocumentContent(obj):
             return super().treatAsEntry(obj)
@@ -263,7 +186,7 @@ class Utilities(web.Utilities):
 
     def inFindContainer(self, obj=None):
         if not obj:
-            obj = orca_state.locusOfFocus
+            obj = focus_manager.get_manager().get_locus_of_focus()
 
         if not obj or self.inDocumentContent(obj):
             return False
@@ -298,35 +221,12 @@ class Utilities(web.Utilities):
             return ""
 
         label = labels[0]
-        AXObject.clear_cache(label)
+        AXObject.clear_cache(label, False, "Ensuring we have correct name for find results.")
         return AXObject.get_name(label)
 
-    def isAutoTextEvent(self, event):
-        if not super().isAutoTextEvent(event):
-            return False
-
-        if self.inDocumentContent(event.source):
-            return True
-
-        if self.treatAsDuplicateEvent(self._lastAutoTextObjectEvent, event) \
-           and time.time() - self._lastAutoTextEventTime < 0.5 \
-           and orca_state.lastInputEvent.isReleaseFor(self._lastAutoTextInputEvent):
-            msg = "GECKO: Event believed to be duplicate auto text event."
-            debug.printMessage(debug.LEVEL_INFO, msg, True)
-            return False
-
-        self._lastAutoTextObjectEvent = event
-        self._lastAutoTextInputEvent = orca_state.lastInputEvent
-        self._lastAutoTextEventTime = time.time()
-        return True
-
     def localizeTextAttribute(self, key, value):
         value = value.replace("-moz-", "")
         return super().localizeTextAttribute(key, value)
 
     def unrelatedLabels(self, root, onlyShowing=True, minimumWords=3):
         return super().unrelatedLabels(root, onlyShowing, minimumWords=1)
-
-    def _shouldUseTableCellInterfaceForCoordinates(self):
-        # https://bugzilla.mozilla.org/show_bug.cgi?id=1794100
-        return False
