@@ -66,11 +66,11 @@ class SpeechServer(speechserver.SpeechServer):
     @staticmethod
     def getSpeechServers():
         servers = []
-        default = SpeechServer._getSpeechServer(SpeechServer.DEFAULT_SERVER_ID)
+        default = SpeechServer._get_speech_server(SpeechServer.DEFAULT_SERVER_ID)
         if default is not None:
             servers.append(default)
             for provider in SpeechServer.DEFAULT_SPEAKER.props.providers:
-                servers.append(SpeechServer._getSpeechServer(provider.props.well_known_name))
+                servers.append(SpeechServer._get_speech_server(provider.props.well_known_name))
         return servers
 
     @classmethod
@@ -107,7 +107,7 @@ class SpeechServer(speechserver.SpeechServer):
         self._current_voice_profiles = voice_profiles
 
     @classmethod
-    def _getSpeechServer(cls, serverId):
+    def _get_speech_server(cls, serverId):
         """Return an active server for given id.
 
         Attempt to create the server if it doesn't exist yet.  Returns None
@@ -121,12 +121,12 @@ class SpeechServer(speechserver.SpeechServer):
         return cls._active_servers.get(serverId)
 
     @staticmethod
-    def getSpeechServer(info):
+    def get_speech_server(info):
         """Gets a given SpeechServer based upon the info.
-        See SpeechServer.getInfo() for more info.
+        See SpeechServer.get_info() for more info.
         """
         thisId = info[1] if info is not None else SpeechServer.DEFAULT_SERVER_ID
-        return SpeechServer._getSpeechServer(thisId)
+        return SpeechServer._get_speech_server(thisId)
 
     @staticmethod
     def shutdownActiveServers():
@@ -260,7 +260,7 @@ class SpeechServer(speechserver.SpeechServer):
                   settings.PUNCTUATION_STYLE_SOME: "SOME",
                   settings.PUNCTUATION_STYLE_MOST: "MOST",
                   settings.PUNCTUATION_STYLE_ALL: "ALL"}
-        manager = settings_manager.getManager()
+        manager = settings_manager.get_manager()
 
         msg = (
             f"SPIEL: {prefix}\n"
@@ -269,7 +269,7 @@ class SpeechServer(speechserver.SpeechServer):
             f"volume {self._current_voice_properties.get(ACSS.GAIN)}, "
             f"language {self._get_language_and_dialect(family)[0]}, "
             f"punctuation: "
-            f"{styles.get(manager.getSetting('verbalizePunctuationStyle'))}\n"
+            f"{styles.get(manager.get_setting('verbalizePunctuationStyle'))}\n"
             f"SD rate {rate}, pitch {pitch}, volume {volume}, language {language}"
         )
         debug.printMessage(debug.LEVEL_INFO, msg, True)
@@ -341,7 +341,7 @@ class SpeechServer(speechserver.SpeechServer):
         self._debug_spiel_values(f"Speaking '{utterance.props.text}' ")
         self._speaker.speak(utterance)
 
-    def getInfo(self):
+    def get_info(self):
         return [self._SERVER_NAMES.get(self._id, self._id), self._id]
 
     def getVoiceFamilies(self):
@@ -385,7 +385,7 @@ class SpeechServer(speechserver.SpeechServer):
 
         return families
 
-    def speakCharacter(self, character, acss=None):
+    def speak_character(self, character, acss=None):
         debug.printMessage(debug.LEVEL_INFO, f"SPIEL Character: '{character}'")
 
         if not acss:
@@ -409,15 +409,15 @@ class SpeechServer(speechserver.SpeechServer):
         utterance = self._create_utterance(text, acss)
         self._speak_utterance(utterance, acss)
 
-    def speakKeyEvent(self, event, acss=None):
-        event_string = event.getKeyName()
-        lockingStateString = event.getLockingStateString()
+    def speak_key_event(self, event, acss=None):
+        event_string = event.get_key_name()
+        lockingStateString = event.get_locking_state_string()
         event_string = f"{event_string} {lockingStateString}".strip()
         if len(event_string) == 1:
             msg = f"SPIEL: Speaking '{event_string}' as key"
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             self._apply_acss(acss)
-            self.speakCharacter(event_string, acss)
+            self.speak_character(event_string, acss)
         else:
             msg = f"SPIEL: Speaking '{event_string}' as string"
             debug.printMessage(debug.LEVEL_INFO, msg, True)
@@ -438,16 +438,16 @@ class SpeechServer(speechserver.SpeechServer):
             msg = f"SPIEL: Speaking '{text}' as char"
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             self._apply_acss(acss)
-            self.speakCharacter(text, acss)
+            self.speak_character(text, acss)
         else:
             msg = f"SPIEL: Speaking '{text}' as string"
             debug.printMessage(debug.LEVEL_INFO, msg, True)
             utterance = self._create_utterance(text, acss)
             self._speak_utterance(utterance, acss)
 
-    def sayAll(self, utteranceIterator, progressCallback):
+    def say_all(self, utterance_iterator, progress_callback):
         try:
-            context, acss = next(utteranceIterator)
+            context, acss = next(utterance_iterator)
         except StopIteration:
             pass
         else:
@@ -466,7 +466,7 @@ class SpeechServer(speechserver.SpeechServer):
                     callback(context, speechserver.SayAllContext.COMPLETED)
                     context.currentOffset = context.endOffset
                     context.currentEndOffset = None
-                    self.sayAll(utteranceIterator, callback)
+                    self.say_all(utterance_iterator, callback)
 
             def _utterance_canceled(speaker, utterance, sayall_data):
                 debug.printMessage(debug.LEVEL_INFO, f"CANCELED: {utterance.props.text}")
@@ -507,17 +507,17 @@ class SpeechServer(speechserver.SpeechServer):
 
             def _sentence_started(_speaker, _utterance, start, end, sayall_data):
                 debug.printMessage(debug.LEVEL_INFO, f"SENTENCE STARTED: {start}-{end}")
-                (callback, currentContext, _) = sayall_data
-                if currentContext == context:
+                (callback, currentUtterance, _) = sayall_data
+                if currentUtterance == utterance:
                     # TODO: map start/end to currentOffset/currentEndOffset
-                    callback(currentContext, speechserver.SayAllContext.PROGRESS)
+                    callback(context, speechserver.SayAllContext.PROGRESS)
 
             utterance = self._create_utterance(context.utterance, acss)
             if not utterance:
                 return
 
             handlers = []
-            sayall_data = (progressCallback, utterance, handlers)
+            sayall_data = (progress_callback, utterance, handlers)
             handlers += [
                 self._speaker.connect('utterance-started', _utterance_started, sayall_data),
                 self._speaker.connect('utterance-finished', _utterance_finished, sayall_data),
