@@ -29,244 +29,269 @@ __license__   = "LGPL"
 
 from . import cmdnames
 from . import debug
+from . import focus_manager
 from . import input_event
 from . import keybindings
 from . import messages
-from . import orca_state
 from . import settings_manager
+from .ax_component import AXComponent
 from .ax_object import AXObject
+from .ax_text import AXText
 from .ax_utilities import AXUtilities
 
-_settingsManager = settings_manager.getManager()
 
 class WhereAmIPresenter:
     """Module for commands related to the current accessible object."""
 
     def __init__(self):
-        self._handlers = self._setup_handlers()
-        self._desktop_bindings = self._setup_desktop_bindings()
-        self._laptop_bindings = self._setup_laptop_bindings()
+        self._handlers = self.get_handlers(True)
+        self._desktop_bindings = keybindings.KeyBindings()
+        self._laptop_bindings = keybindings.KeyBindings()
 
-    def get_bindings(self, is_desktop):
+    def get_bindings(self, refresh=False, is_desktop=True):
         """Returns the where-am-i-presenter keybindings."""
 
+        if refresh:
+            msg = "WHERE AM I PRESENTER: Refreshing bindings."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            self._setup_bindings()
+        elif is_desktop and self._desktop_bindings.is_empty():
+            self._setup_bindings()
+        elif not is_desktop and self._laptop_bindings.is_empty():
+            self._setup_bindings()
+
         if is_desktop:
             return self._desktop_bindings
         return self._laptop_bindings
 
-    def get_handlers(self):
+    def get_handlers(self, refresh=False):
         """Returns the where-am-i-presenter handlers."""
 
+        if refresh:
+            msg = "WHERE AM I PRESENTER: Refreshing handlers."
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
+            self._setup_handlers()
+
         return self._handlers
 
+    def _setup_bindings(self):
+        """Sets up the where-am-i-presenter key bindings."""
+
+        self._setup_desktop_bindings()
+        self._setup_laptop_bindings()
+
     def _setup_handlers(self):
-        """Sets up and returns the where-am-i-presenter input event handlers."""
+        """Sets up the where-am-i-presenter input event handlers."""
 
-        handlers = {}
+        self._handlers = {}
 
-        handlers["readCharAttributesHandler"] = \
+        self._handlers["readCharAttributesHandler"] = \
             input_event.InputEventHandler(
                 self.present_character_attributes,
                 cmdnames.READ_CHAR_ATTRIBUTES)
 
-        handlers["presentSizeAndPositionHandler"] = \
+        self._handlers["presentSizeAndPositionHandler"] = \
             input_event.InputEventHandler(
                 self.present_size_and_position,
                 cmdnames.PRESENT_SIZE_AND_POSITION)
 
-        handlers["getTitleHandler"] = \
+        self._handlers["getTitleHandler"] = \
             input_event.InputEventHandler(
                 self.present_title,
                 cmdnames.PRESENT_TITLE)
 
-        handlers["getStatusBarHandler"] = \
+        self._handlers["getStatusBarHandler"] = \
             input_event.InputEventHandler(
                 self.present_status_bar,
                 cmdnames.PRESENT_STATUS_BAR)
 
-        handlers["present_default_button"] = \
+        self._handlers["present_default_button"] = \
             input_event.InputEventHandler(
                 self.present_default_button,
                 cmdnames.PRESENT_DEFAULT_BUTTON)
 
-        handlers["whereAmIBasicHandler"] = \
+        self._handlers["whereAmIBasicHandler"] = \
             input_event.InputEventHandler(
                 self.where_am_i_basic,
                 cmdnames.WHERE_AM_I_BASIC)
 
-        handlers["whereAmIDetailedHandler"] = \
+        self._handlers["whereAmIDetailedHandler"] = \
             input_event.InputEventHandler(
                 self.where_am_i_detailed,
                 cmdnames.WHERE_AM_I_DETAILED)
 
-        handlers["whereAmILinkHandler"] = \
+        self._handlers["whereAmILinkHandler"] = \
             input_event.InputEventHandler(
                 self.present_link,
                 cmdnames.WHERE_AM_I_LINK)
 
-        handlers["whereAmISelectionHandler"] = \
+        self._handlers["whereAmISelectionHandler"] = \
             input_event.InputEventHandler(
                 self.present_selection,
                 cmdnames.WHERE_AM_I_SELECTION)
 
-        return handlers
+        msg = "WHERE AM I PRESENTER: Handlers set up."
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _setup_desktop_bindings(self):
-        """Sets up and returns the where-am-i-presenter desktop key bindings."""
+        """Sets up the where-am-i-presenter desktop key bindings."""
 
-        bindings = keybindings.KeyBindings()
+        self._desktop_bindings = keybindings.KeyBindings()
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "f",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("readCharAttributesHandler")))
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "e",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("present_default_button")))
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
                 self._handlers.get("presentSizeAndPositionHandler")))
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "KP_Enter",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("getTitleHandler"),
                 1))
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "KP_Enter",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("getStatusBarHandler"),
                 2))
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "KP_Enter",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
                 self._handlers.get("whereAmIBasicHandler"),
                 1))
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "KP_Enter",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
                 self._handlers.get("whereAmIDetailedHandler"),
                 2))
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
                 self._handlers.get("whereAmILinkHandler")))
 
-        bindings.add(
+        self._desktop_bindings.add(
             keybindings.KeyBinding(
                 "Up",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_SHIFT_MODIFIER_MASK,
                 self._handlers.get("whereAmISelectionHandler")))
 
-        return bindings
+        msg = "WHERE AM I PRESENTER: Desktop bindings set up."
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def _setup_laptop_bindings(self):
-        """Sets up and returns the where-am-i-presenter laptop key bindings."""
+        """Sets up the where-am-i-presenter laptop key bindings."""
 
-        bindings = keybindings.KeyBindings()
+        self._laptop_bindings = keybindings.KeyBindings()
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "f",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("readCharAttributesHandler")))
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "e",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("present_default_button")))
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
                 self._handlers.get("presentSizeAndPositionHandler")))
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "slash",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("getTitleHandler"),
                 1))
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "slash",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("getStatusBarHandler"),
                 2))
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "Return",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("whereAmIBasicHandler"),
                 1))
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "Return",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_MODIFIER_MASK,
                 self._handlers.get("whereAmIDetailedHandler"),
                 2))
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.NO_MODIFIER_MASK,
                 self._handlers.get("whereAmILinkHandler")))
 
-        bindings.add(
+        self._laptop_bindings.add(
             keybindings.KeyBinding(
                 "Up",
-                keybindings.defaultModifierMask,
+                keybindings.DEFAULT_MODIFIER_MASK,
                 keybindings.ORCA_SHIFT_MODIFIER_MASK,
                 self._handlers.get("whereAmISelectionHandler")))
 
-        return bindings
+        msg = "WHERE AM I PRESENTER: Laptop bindings set up."
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     def present_character_attributes(self, script, event=None):
         """Presents the font and formatting details for the current character."""
 
-        attrs = script.utilities.textAttributes(orca_state.locusOfFocus, None, True)[0]
+        focus = focus_manager.get_manager().get_locus_of_focus()
+        attrs = AXText.get_text_attributes_at_offset(focus)[0]
 
         # Get a dictionary of text attributes that the user cares about.
         [user_attr_list, user_attr_dict] = script.utilities.stringToKeysAndDict(
-            _settingsManager.getSetting('enabledSpokenTextAttributes'))
+            settings_manager.get_manager().get_setting('enabledSpokenTextAttributes'))
 
         null_values = ['0', '0mm', 'none', 'false']
         for key in user_attr_list:
@@ -285,35 +310,35 @@ class WhereAmIPresenter:
     def present_size_and_position(self, script, event=None):
         """Presents the size and position of the current object."""
 
-        if script.flatReviewPresenter.is_active():
-            obj = script.flatReviewPresenter.get_current_object(script, event)
+        if script.get_flat_review_presenter().is_active():
+            obj = script.get_flat_review_presenter().get_current_object(script, event)
         else:
-            obj = orca_state.locusOfFocus
+            obj = focus_manager.get_manager().get_locus_of_focus()
 
-        x_coord, y_coord, width, height = script.utilities.getBoundingBox(obj)
-        if (x_coord, y_coord, width, height) == (-1, -1, 0, 0):
+        rect = AXComponent.get_rect(obj)
+        if AXComponent.is_empty_rect(rect):
             full = messages.LOCATION_NOT_FOUND_FULL
             brief = messages.LOCATION_NOT_FOUND_BRIEF
             script.presentMessage(full, brief)
             return True
 
-        full = messages.SIZE_AND_POSITION_FULL % (width, height, x_coord, y_coord)
-        brief = messages.SIZE_AND_POSITION_BRIEF % (width, height, x_coord, y_coord)
+        full = messages.SIZE_AND_POSITION_FULL % (rect.width, rect.height, rect.x, rect.y)
+        brief = messages.SIZE_AND_POSITION_BRIEF % (rect.width, rect.height, rect.x, rect.y)
         script.presentMessage(full, brief)
         return True
 
     def present_title(self, script, event=None):
         """Presents the title of the current window."""
 
-        obj = orca_state.locusOfFocus
+        obj = focus_manager.get_manager().get_locus_of_focus()
         if AXObject.is_dead(obj):
-            obj = orca_state.activeWindow
+            obj = focus_manager.get_manager().get_active_window()
 
         if obj is None or AXObject.is_dead(obj):
             script.presentMessage(messages.LOCATION_NOT_FOUND_FULL)
             return True
 
-        title = script.speechGenerator.generateTitle(obj)
+        title = script.speech_generator.generate_window_title(obj)
         for (string, voice) in title:
             script.presentMessage(string, voice=voice)
         return True
@@ -321,7 +346,7 @@ class WhereAmIPresenter:
     def _present_default_button(self, script, event=None, dialog=None, error_messages=True):
         """Presents the default button of the current dialog."""
 
-        obj = orca_state.locusOfFocus
+        obj = focus_manager.get_manager().get_locus_of_focus()
         frame, dialog = script.utilities.frameAndDialog(obj)
         if dialog is None:
             if error_messages:
@@ -345,14 +370,12 @@ class WhereAmIPresenter:
     def present_status_bar(self, script, event=None):
         """Presents the status bar of the current window."""
 
-        obj = orca_state.locusOfFocus
+        obj = focus_manager.get_manager().get_locus_of_focus()
         frame, dialog = script.utilities.frameAndDialog(obj)
         if frame:
             statusbar = AXUtilities.get_status_bar(frame)
             if statusbar:
-                script.pointOfReference['statusBarItems'] = None
                 script.presentObject(statusbar, interrupt=True)
-                script.pointOfReference['statusBarItems'] = None
             else:
                 full = messages.STATUS_BAR_NOT_FOUND_FULL
                 brief = messages.STATUS_BAR_NOT_FOUND_BRIEF
@@ -376,7 +399,7 @@ class WhereAmIPresenter:
     def present_link(self, script, event=None, link=None):
         """Presents details about the current link."""
 
-        link = link or orca_state.locusOfFocus
+        link = link or focus_manager.get_manager().get_locus_of_focus()
         if not script.utilities.isLink(link):
             script.presentMessage(messages.NOT_ON_A_LINK)
             return True
@@ -386,7 +409,7 @@ class WhereAmIPresenter:
     def present_selected_text(self, script, event=None, obj=None):
         """Presents the selected text."""
 
-        obj = obj or orca_state.locusOfFocus
+        obj = obj or focus_manager.get_manager().get_locus_of_focus()
         if obj is None:
             script.speakMessage(messages.LOCATION_NOT_FOUND_FULL)
             return True
@@ -406,7 +429,7 @@ class WhereAmIPresenter:
     def present_selection(self, script, event=None, obj=None):
         """Presents the selected text or selected objects."""
 
-        obj = obj or orca_state.locusOfFocus
+        obj = obj or focus_manager.get_manager().get_locus_of_focus()
         if obj is None:
             script.speakMessage(messages.LOCATION_NOT_FOUND_FULL)
             return True
@@ -442,9 +465,9 @@ class WhereAmIPresenter:
             script.spellcheck.presentErrorDetails(not basic_only)
 
         if obj is None:
-            obj = orca_state.locusOfFocus
+            obj = focus_manager.get_manager().get_locus_of_focus()
         if AXObject.is_dead(obj):
-            obj = orca_state.activeWindow
+            obj = focus_manager.get_manager().get_active_window()
 
         if obj is None or AXObject.is_dead(obj):
             script.presentMessage(messages.LOCATION_NOT_FOUND_FULL)
@@ -482,7 +505,7 @@ class WhereAmIPresenter:
         return self._do_where_am_i(script, event, False)
 
 _presenter = WhereAmIPresenter()
-def getPresenter():
+def get_presenter():
     """Returns the Where Am I Presenter"""
 
     return _presenter
